{% extends "admin/change_form.html" %}

{% block object-tools %}
    <p>
    Download the empty <a href="?download_template=1">template</a>, fill it out, then upload. See user manual for more information.
    </p>
    {{ block.super }}
{% endblock %}
