# Generated by Django 3.2.7 on 2022-02-06 17:37

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("music_publisher", "0005_auto_20220206_1720"),
    ]

    operations = [
        migrations.AlterField(
            model_name="cwrexport",
            name="nwr_rev",
            field=models.CharField(
                choices=[
                    ("NW2", "CWR 2.2: New work registrations"),
                    ("RE2", "CWR 2.2: Revisions of registered works"),
                ],
                db_index=True,
                default="NWR",
                max_length=3,
                verbose_name="CWR version/type",
            ),
        ),
    ]
